from django.contrib import admin


class Select2FilterMixin:
    select2_widget_class = None
    dependent_fields = {}

    def init_select2(self):
        if self.select2_widget_class is None:
            raise ValueError('You must specify "select2_widget_class".')

    def has_output(self):
        return True

    def queryset(self, request, queryset):
        val = self.value()
        if val:
            return queryset.filter(**{self.parameter_name: val})
        return queryset


class MultipleSelect2FilterMixin:
    multiple = True

    def value_as_list(self):
        return self.value().split(',') if self.value() else []

    def queryset(self, request, queryset):
        val = self.value_as_list()
        if len(val) > 0:
            return queryset.filter(**{f'{self.parameter_name}__in': val})
        return queryset


class ModelFilterMixin(Select2FilterMixin):
    autocomplete_queryset = None
    search_fields = None

    def init_select2(self):
        super().init_select2()
        if self.autocomplete_queryset is None:
            raise ValueError('You must specify "autocomplete_queryset".')
        if self.search_fields is None:
            raise ValueError('You must specify "search_fields".')
        select2 = self.select2_widget_class(
            queryset=self.autocomplete_queryset,
            search_fields=self.search_fields,
            dependent_fields=self.dependent_fields,
        )
        select2.set_to_cache()
        self.attrs = select2.build_attrs(base_attrs={
            'id': f'id-{self.parameter_name}-filter',
            'name': f'{self.parameter_name}-filter',
            'data-parameter_name': self.parameter_name,
        })
        self.css_class = self.attrs.pop('class', '')

    def get_display_label(self, obj):
        return str(obj)

    def lookups(self, request, model_admin):
        return []


class ChoiceFilterMixin(Select2FilterMixin):
    autocomplete_choice_list = None

    def init_select2(self):
        super().init_select2()
        if self.autocomplete_choice_list is None:
            raise ValueError('You must specify "autocomplete_choice_list".')
        select2 = self.select2_widget_class(
            choices=self.autocomplete_choice_list,
        )
        self.attrs = select2.build_attrs(base_attrs={
            'id': f'id-{self.parameter_name}-filter',
            'name': f'{self.parameter_name}-filter',
            'data-parameter_name': self.parameter_name,
        })
        self.css_class = self.attrs.pop('class', '')

    def lookups(self, request, model_admin):
        return self.autocomplete_choice_list


class AdminFilterWidget(admin.SimpleListFilter):
    template = 'admin/select2_filter.html'
    parameter_name = None

    def __init__(self, request, params, model, model_admin):
        super().__init__(request, params, model, model_admin)
        self.init_select2()



from django_select2.forms import (
    ModelSelect2MultipleWidget, ModelSelect2Widget, Select2MultipleWidget,
    Select2Widget)

from .mixins import (
    AdminFilterWidget, ChoiceFilterMixin, ModelFilterMixin,
    MultipleSelect2FilterMixin)


class ModelSelect2Filter(ModelFilterMixin, AdminFilterWidget):
    select2_widget_class = ModelSelect2Widget

    def get_selected(self):
        val = self.value()
        if val:
            return self.autocomplete_queryset.filter(pk=val)
        return None

    def choices(self, changelist):
        yield {
            'selected': self.value() is None,
            'value': '',
            'display': '',
        }
        for obj in self.get_selected():
            yield {
                'selected': True,
                'value': obj.pk,
                'display': self.get_display_label(obj),
            }


class MultipleModelSelect2Filter(
    ModelFilterMixin,
    MultipleSelect2FilterMixin,
    AdminFilterWidget,
):
    select2_widget_class = ModelSelect2MultipleWidget

    def get_selected(self):
        val = self.value_as_list()
        if val:
            return self.autocomplete_queryset.filter(pk__in=val)
        return []

    def choices(self, changelist):
        for obj in self.get_selected():
            yield {
                'selected': True,
                'value': obj.pk,
                'display': self.get_display_label(obj),
            }


class ChoiceSelect2Filter(ChoiceFilterMixin, AdminFilterWidget):
    select2_widget_class = Select2Widget

    def choices(self, changelist):
        selected = self.value()
        yield {
            'selected': self.value() is None,
            'value': '',
            'display': '',
        }
        for val, name in self.autocomplete_choice_list:
            yield {
                'selected': str(val) == selected,
                'value': val,
                'display': name,
            }


class MultipleChoiceSelect2Filter(
    ChoiceFilterMixin,
    MultipleSelect2FilterMixin,
    AdminFilterWidget,
):
    select2_widget_class = Select2MultipleWidget

    def choices(self, changelist):
        selected = [str(val) for val in self.value_as_list()]
        for val, name in self.autocomplete_choice_list:
            yield {
                'selected': val in selected,
                'value': val,
                'display': name,
            }



from django.conf import settings

if 'grappelli' in settings.INSTALLED_APPS:
    TEMPLATE = 'admin/select2_change_list_filter_confirm_sidebar.html'
else:
    TEMPLATE = 'admin/select2_change_list.html'


class Select2AdminFilterMixin:
    change_list_template = TEMPLATE



from django import template
from django.template.loader import get_template

register = template.Library()


@register.simple_tag
def admin_list_filter(cl, spec):
    try:
        tpl = get_template(cl.model_admin.change_list_filter_template)
    except:
        tpl = get_template(spec.template)
    return tpl.render({
        'title': spec.title,
        'choices': list(spec.choices(cl)),
        'spec': spec,
        'class': spec.css_class,
        'attrs': spec.attrs,
        'multiple': getattr(spec, 'multiple', False),
    })



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import re
import shutil
import sys
from io import open

from setuptools import find_packages, setup


def read(f):
    return open(f, 'r', encoding='utf-8').read()


def get_version(package):
    """
    Return package version as listed in `__version__` in `__init__.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('django_select2_admin_filters')


setup(
    name='django-select2-admin-filters',
    version=version,
    license='MIT',
    description='A simple extension to Django app to render filters in django admin panel as autocomplete widget.',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    author='Bartłomiej Żmudziński',
    author_email='zmudzinski.bartek@gmail.com',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    install_requires=["django>=2.2"],
    python_requires=">=3.5",
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3 :: Only',
        'Operating System :: OS Independent',
    ],
    project_urls={
        'Source': 'https://github.com/the-bit-by-bit/django-select2-admin-filters',
    },
)